﻿require ("prototypes.scripts.defines")

local mod_gui = require("__core__/lualib/mod-gui")

local is_valid  = modmashsplinterboom.util.is_valid
local print = modmashsplinterboom.util.print

local local_init = function()	
	if storage.modmashsplinterboom.landmines == nil then storage.modmashsplinterboom.landmines = {} end	
	if storage.modmashsplinterboom.landmines.highlights == nil then storage.modmashsplinterboom.landmines.highlights = {} end
	end

local local_create_element = function(parent,type,name,caption,tooltip)
	if not (parent and parent.valid) then return nil end
	local element = {}
	element.type    = type
	element.name    = name
	element.caption = caption
	element.tooltip = tooltip
	return element
	end
	
local local_add_sprite_button = function(parent, name, sprite, tooltip, style)
	local element = local_create_element(parent,"sprite-button", name, nil,tooltip)
	if element == nil then return nil end
	element.style = style or "button"
	element.sprite = sprite
	return parent.add(element)
	end

local local_update_shortcut_gui = function()
	local inner_update_by_surface = function(player)
		local surface = player.surface

		local any = false
		for k=1 , #storage.modmashsplinterboom.landmines do local lm = storage.modmashsplinterboom.landmines[k]
			if is_valid(lm) and lm.surface == player.surface then
				any = true
				break
			end			
		end
		if any == true then
			player.set_shortcut_available("boom-toggle",true)
		else
			player.set_shortcut_available("boom-toggle",false)
		end
	end

	local inner_update = function(player)
		if #storage.modmashsplinterboom.landmines > 0 then
			player.set_shortcut_available("boom-all-toggle",true)
		else
			player.set_shortcut_available("boom-all-toggle",false)
		end
	end
	for k=1 , #game.players do
		local player = game.players[k]
		if is_valid(player) == true then
			inner_update_by_surface(player)
			inner_update(player)
		end
	end	
	end

local local_update_mod_gui = function()
	local inner_update_by_surface = function(pi)
		local player = game.players[pi]
		local surface = game.players[pi].surface

		local button_flow =  mod_gui.get_button_flow(player)
		local any = false
		for k=1 , #storage.modmashsplinterboom.landmines do local lm = storage.modmashsplinterboom.landmines[k]
			if is_valid(lm) and lm.surface == player.surface then
				any = true
				break
			end			
		end
		if any == true then
			for k = #button_flow.children, 1, -1 do
				local button = button_flow.children[k]
				if button and button.name == "landmine-toggle-button" then
					return
				end
			end
			local_add_sprite_button(button_flow, "landmine-toggle-button","landmine-button-gui",{"gui.modmash-landmine-detonate-tooltip"},"landmine-icon-button")
		else
			for k = #button_flow.children, 1, -1 do
				local button = button_flow.children[k]
				if button and button.name == "landmine-toggle-button" then
					button.destroy()
					return
				end
			end
		end
	end

	local inner_update = function(pi)
		local button_flow =  mod_gui.get_button_flow(game.players[pi])
		
		if #storage.modmashsplinterboom.landmines > 0 then
			for k = #button_flow.children, 1, -1 do
				local button = button_flow.children[k]
				if button and button.name == "landmine-toggle-button" then
					return
				end
			end
			local_add_sprite_button(button_flow, "landmine-toggle-button","landmine-button-gui",{"gui.modmash-landmine-detonate-tooltip"},"landmine-icon-button")
		else
			for k = #button_flow.children, 1, -1 do
				local button = button_flow.children[k]
				if button and button.name == "landmine-toggle-button" then
					button.destroy()
					return
				end
			end
		end
	end
	if settings.storage["setting-shortcuts-only"].value == false then
		for k=1 , #game.players do
			local flow = mod_gui.get_button_flow(game.players[k])
			for j = #flow.children, 1, -1 do
				local button = flow.children[j]
				if button and button.name == "landmine-toggle-button" then
					button.destroy()
					break
				end
			end
		end
	else
		if settings.storage["setting-surface-detination"].value == "Current" then
			for k=1 , #game.players do
				inner_update_by_surface(k)
			end	
		else
			for k=1 , #game.players do
				inner_update(k)
			end	
		end
	end
	end

local local_update_gui = function()
	local_update_shortcut_gui()
	local_update_mod_gui()
end

local local_landmine_added = function(entity)
	if entity.type == "land-mine"  then		
		table.insert(storage.modmashsplinterboom.landmines, entity)		
		local_update_gui()
	end	
end

local local_landmine_removed = function(entity)
	if entity.type == "land-mine"  then				
		for k=0, #storage.modmashsplinterboom.landmines do local landmine = storage.modmashsplinterboom.landmines[k]
			if  landmine == entity then
				table.remove(storage.modmashsplinterboom.landmines, index)		
				local_update_gui()
				return
			end
		end
	end	
	local_update_gui()
end

local local_damage_table = {}
local_damage_table["land-mine"] = {damage=250,area=1.75}
local_damage_table["nuclear-land-mine"] = {damage=400,area=4}


local local_get_damage = function(landmine)
	if local_damage_table[landmine.name] ~= nil then return local_damage_table[landmine.name] end
	return {damage=250,area=1.75}
end

local local_do_boom = function(player,all)
	
	local copy = {}
	for k=1 , #storage.modmashsplinterboom.landmines do
		table.insert(copy,storage.modmashsplinterboom.landmines[k])
	end
	storage.modmashsplinterboom.landmines = {}

	if all == false then
		local surface = player.surface
		for k = 1, #copy do local landmine = copy[k]	
			if is_valid(landmine) == true then
				if landmine.surface == surface then
					local p = landmine.position
					local s = landmine.surface
					local d = local_get_damage(landmine)
					landmine.die()
				else
					table.insert(storage.modmashsplinterboom.landmines,landmine)
				end
			end
		end
	else
		for k = 1, #copy do local landmine = copy[k]
			
			if is_valid(landmine) then
				local p = landmine.position
				local s = landmine.surface
				local d = local_get_damage(landmine)
				landmine.die()
			end
		end
	end
	local_update_gui()
end

local local_landmine_on_shortcut_gui_click = function(event)
	local player = game.players[event.player_index]

	if event.prototype_name == "boom-toggle" and player.is_shortcut_available("boom-toggle") then
		local_do_boom(player,false)
	elseif event.prototype_name == "boom-all-toggle" and player.is_shortcut_available("boom-all-toggle") then
		local_do_boom(player,true)
	end
	end


local local_landmine_on_gui_click = function(event)
	if event.element.name == "landmine-toggle-button" then
		local player = game.players[event.player_index]
		if settings.storage["setting-surface-detination"].value == "Current" then
			local_do_boom(player,false)
		else
			local_do_boom(player,true)
		end
	end
	end

local local_on_player_changed_surface = function(event)	
	local_update_gui()
	end
	

local local_on_entity_selected = function(event)
	local player = game.players[event.player_index]
	local entity = player.selected	
	local player_index = player.index
	if storage.modmashsplinterboom.landmines.highlights == nil then storage.modmashsplinterboom.landmines.highlights = {} end
	if is_valid(entity) and entity.name == "nuclear-land-mine" then		
		if storage.modmashsplinterboom.landmines.highlights[player_index] ~= nil then
			rendering.destroy(storage.modmashsplinterboom.landmines.highlights[player_index])
		end
		local id = rendering.draw_circle
		{
			surface = entity.surface,
			players = {player},
			filled = true,
			color = {r = 1, g = 0.1, b = 0, a = 0.1},
			draw_on_ground = true,
			width = 36,
			filled = false,
			target = entity,
			only_in_alt_mode = false,
			radius = 36,
			time_to_live = 60*10
		}
		storage.modmashsplinterboom.landmines.highlights[player_index] = id
	elseif storage.modmashsplinterboom.landmines.highlights[player_index] ~= nil then
		rendering.destroy(storage.modmashsplinterboom.landmines.highlights[player_index])
	end
end


local local_on_runtime_mod_setting_changed = function(event)
	local_update_gui()
	end


local local_on_configuration_changed = function() 	
	local_update_gui()
	end


local local_land_mine_filter = { {filter = "type", type = "land-mine" }}
script.on_init(local_init)
script.on_load(local_load)

script.on_configuration_changed(local_on_configuration_changed)
script.on_event(defines.events.on_runtime_mod_setting_changed,local_on_runtime_mod_setting_changed)

script.on_event(defines.events.on_player_changed_surface,local_on_player_changed_surface)

script.on_event(defines.events.on_selected_entity_changed,local_on_entity_selected)

script.on_event(defines.events.on_entity_died,
	function(event) 
		if is_valid(event.entity) then local_landmine_removed(event.entity) end 
	end,local_land_mine_filter)

script.on_event(defines.events.on_robot_mined_entity,
	function(event) 
		if is_valid(event.entity) then local_landmine_removed(event.entity) end 
	end,local_land_mine_filter)

script.on_event(defines.events.on_player_mined_entity,
	function(event) 
		if is_valid(event.entity) then local_landmine_removed(event.entity) end 
	end,local_land_mine_filter)
script.on_event(defines.events.script_raised_revive,
	function(event) 
		if is_valid(event.entity) then local_landmine_added(event.entity) end 
	end,local_land_mine_filter)
script.on_event(defines.events.on_robot_built_entity,
	function(event) 
		if is_valid(event.created_entity) then local_landmine_added(event.created_entity) end 
	end,local_land_mine_filter)
script.on_event(defines.events.on_built_entity,
	function(event) 
		if is_valid(event.created_entity) then local_landmine_added(event.created_entity) end 
	end,local_land_mine_filter)
script.on_event(defines.events.script_raised_built,
	function(event) 
		if is_valid(event.created_entity) then local_landmine_added(event.entity) end 
	end,local_land_mine_filter)

script.on_event(defines.events.on_lua_shortcut, local_landmine_on_shortcut_gui_click)

script.on_event(defines.events.on_gui_click, local_landmine_on_gui_click)

